% MCMCSearchScaleFactor
%
% Searches for the good scale factor for the MCMC jump distribution
%
% Required settings:
%
% Optional settings:
%
%   UseParallel
%   whether to use parallel or not. Default: false
% 
%   nChains
%   Sets the number of MCMC chains to use in each search block.
%   Default: 4
%
%   nDrawsSearch
%   number of MCMC draws per chain
%
%   ScaleJumpFactor
%   Initial factor. 
%   Default: 2.4
%
%   nIRS
%   size of importance resample. Default: 2000
%
%   dscale
%   double array with alternative scale changes to be tried in succession 
%   until proper rejection rate is confirmed.
%   Default: [0.2,0.05,0.01]
%
%   nConfirm
%   number of times the rejection rate in range needs to be confirmed.
%   Default: 1
%
%   RejectionRateMax
%   Maximum rejection rate
%   Default: 0.80
%
%   RejectionRateMin
%   Minimum rejection rate
%   Default: 0.70
%
%   MaxReverseDirection
%   Maximum number of times that the direction of search can be reverted
%   before attempting to change scale or simply terminate search.
%   Default: 2
%
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost,
% MaxPostFcn, MakeTableMaxPost, MCMC, MCMCFcn, MCMCSearchScaleFactor,
% MakePlotsMCMCDraws, MCMCInference, MakeTableMCMCInference, 
% MakePlotsMCMCTrace, MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv,
% MakeReportMCMCPlots, MakeReportMCMC
%
% .........................................................................
%
% Created: March 31, 2008
% Updated: August 25, 2010
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% Settings
if ~exist('UseParallel','var'),UseParallel = 0; end
if ~exist('nChains','var'), nChains = 4; end
if ~exist('nDrawsSearch','var'), nDrawsSearch = 2000; end
if ~exist('nUpdate','var'), nUpdate = 0; end
if exist('nIRS','var'), MCMCOptions.nIRS = nIRS; end
FileName.MCMCDrawsSSF = sprintf('%sMCMCDrawsSSFUpdate%.0fChain',FileName.Output,nUpdate); 
% if ~exist('MCMCSSFLogFileName','var'), MCMCSSFLogFileName = ''; end
MCMCSSFLogFileName = sprintf('%sMCMCDrawsSSFUpdate%.0fChain',FileName.Output,nUpdate);

%% settings for parallel computing
if ~exist('UseParallel','var'),UseParallel = 0; end
if ~exist('nMaxWorkers','var'), nMaxWorkers = 4; end

%% settings regarding the scale search
if ~exist('ScaleJumpFactor','var'), ScaleJumpFactor = 2.4; end
if ~exist('dscale','var'), dscale = [0.2,0.05,0.01]; end
if ~exist('nConfirm','var'), nConfirm = 1; end
if ~exist('RejectionRateMax','var'), RejectionRateMax = 0.80; end
if ~exist('RejectionRateMin','var'), RejectionRateMin = 0.70; end
if ~exist('MaxReverseDirection','var'), MaxReverseDirection = 2; end

%% ------------------------------------------------------------------------

%% Display
fprintf('\n**************************************************')
fprintf('\n* Search Scale Factor for MCMC jump distribution *')
fprintf('\n**************************************************\n\n')

%% Set Timer
TimeStr = sprintf('MCMCSearchScaleFactorUpdate%.0f',nUpdate);
TimeElapsed.(TimeStr) = toc;

%% Search for scale factor
jConfirm = 0;
jscale = 1;
nRevertDirection = 0;
currChange = 0;
nBlocks = 0;
while jConfirm<=nConfirm %% loop needs fixing!!!!
    nBlocks = nBlocks+1;
%     BlockFileName = sprintf('%sb%02.0fc',FileName.MCMCDrawsSSF,nBlocks);
    % update ScaleJumpFactor
    MCMCOptions.ScaleJumpFactor = ScaleJumpFactor;
    % simulate MCMC
    if ~UseParallel
        for j=1:nChains
            fprintf('\nGenerating Chain %.0f of MCMC draws...\n',j)
            MCMCFcn(FileName.Post,sprintf('%s%.0f',FileName.MCMCDrawsSSF,j),Post,nDrawsSearch,MCMCOptions);
        end
    else
        JobOptions = cell(nChains,1);
        for jChain=1:nChains
            JobOptions{jChain} = {FileName.Post,...
                sprintf('%s%.0f',FileName.MCMCDrawsSSF,jChain),Post,nDrawsSearch,MCMCOptions};
        end
        vcSubmitQueuedJobs(nChains,@MCMCFcn,0,JobOptions,ListPathDependencies,nMaxWorkers,...
            'nMaxTasks',nMaxWorkers,'ShowOutput',0,...
            'SaveTmpFileName',[FileName.Output,'_MCMCSearchScaleFactorTmp'],...
            'LogFileName',MCMCSSFLogFileName);
        clear JobOptions
    end
    % display rejection rates
    fprintf('\nLast chain results:\n')
    nAbove = 0;
    nBelow = 0;
    RejectionRates(nBlocks,:) = zeros(1,nChains);
    for jChain=1:nChains
        eval(sprintf('load %s%.0f nRejections',FileName.MCMCDrawsSSF,jChain))
        eval(sprintf('delete %s%.0f.mat',FileName.MCMCDrawsSSF,jChain))
        RejectionRates(nBlocks,jChain) = nRejections/nDrawsSearch;
        ScaleJumpFactors(nBlocks) = ScaleJumpFactor;
        fprintf('Block %03.0f, Chain %02.0f: ScaleJumpFactor = %4.2f, rejection rate = %5.1f%%\n',...
            nBlocks,jChain,ScaleJumpFactor,RejectionRates(nBlocks,jChain)*100)
        nAbove = nAbove + (RejectionRates(nBlocks,jChain)>RejectionRateMax);
        nBelow = nBelow + (RejectionRates(nBlocks,jChain)<RejectionRateMin);
    end
    fprintf('\nNumber of chains with rejection rate too high: %.0f',nAbove)
    fprintf('\nNumber of chains with rejection rate too low: %.0f\n\n',nBelow)
    if nAbove==nBelow
        jConfirm = jConfirm+1;
        if jConfirm>nConfirm
            fprintf('Results confirmed!\n')
        else
            fprintf('Rejection rate in range. Confirming results...\n')
        end
    else
        lastChange = currChange;
        currChange = (nAbove<nBelow)-(nAbove>nBelow);
        fprintf('Direction: %.0f\n',currChange)
        if lastChange+currChange==0
            nRevertDirection = nRevertDirection+1;
            fprintf('Direction reverted %.0f times\n',nRevertDirection)
        end
        if nRevertDirection>MaxReverseDirection
            fprintf('Direction reverted too many times. Changing increments...\n')
            jscale = jscale+1;
            nRevertDirection=0;
        end
        if jscale>length(dscale)
            jConfirm = jConfirm+1;
            if jConfirm>nConfirm
                fprintf('Results confirmed!\n')
            else
                fprintf('Increments changed too many times. Confirming results...\n')
            end
        else
            ScaleJumpFactor = ScaleJumpFactor +currChange*dscale(jscale);
            fprintf('Scale changed to %.2f\n',ScaleJumpFactor)
            jConfirm = 0;
        end            
    end
end

%% Show rejection rates:
fprintf('\nResults for ScaleJumpSearch:\n\n')
for j=1:nBlocks
    for jj=1:nChains
        fprintf('Block %03.0f, Chain %02.0f: ScaleJumpFactor = %4.2f, rejection rate = %4.1f%%\n',...
            j,jj,ScaleJumpFactors(j),RejectionRates(j,jj)*100)
    end
    fprintf('\n')
end

%% show reason to stop
fprintf('\nReason to stop: ')
if jscale>length(dscale)
    fprintf('Increments changed too many times.\n\n')
else
    fprintf('Results confirmed.\n\n')
end

%% Clean up
if ~isempty(MCMCSSFLogFileName)
    for jChain=1:nChains
        delete(sprintf('%s%03.0f.log',MCMCSSFLogFileName,jChain));
    end
end
clear dscale jscale jBlock jChain

%% Show time taken
TimeElapsed.(TimeStr) = toc-TimeElapsed.(TimeStr);
fprintf('\n%s %s\n\n',TimeStr,vctoc([],TimeElapsed.(TimeStr)))

%% ------------------------------------------------------------------------
