% MCMC
%
% Runs several chains of MCMC
%
% Required settings:
%
% Optional settings:
%
%   UseParallel
%   whether to use parallel or not. Default: false
% 
%   nChains
%   Sets the number of MCMC chains.
%   Default: 4
%
%   nDraws
%   number of MCMC draws per chain
%
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost, 
% MakeTableMaxPost, MCMC, MCMCSearchScaleFactor, MakePlotsMCMCConv, 
% MCMCInference, MakeTableMCMCInference, MakePlotsMCMCTrace, 
% MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv, MCMCVD
%
% .........................................................................
%
% Created: March 27, 2008
% Updated: August 25, 2010
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% Settings
if ~exist('nChains','var'), nChains = 4; end
if ~exist('nDraws','var'), nDraws = 50000; end
if ~exist('nUpdate','var'), nUpdate = 0; end
FileName.MCMCDraws = sprintf('%sMCMCDrawsUpdate%.0fChain',FileName.Output,nUpdate); 
% if ~exist('MCMCLogFileName','var'), MCMCLogFileName = ''; end
MCMCLogFileName = sprintf('%sMCMCDrawsUpdate%.0fChain',FileName.Output,nUpdate);

%% settings for parallel computing
if ~exist('UseParallel','var'),UseParallel = 0; end
if ~exist('nMaxWorkers','var'), nMaxWorkers = 4; end
if ~exist('IsMakeLogs','var'), IsMakeLogs = 0; end

%% Prepare options
if exist('MCMCOptions','var')
    options{1} = MCMCOptions;
else
    options = {};
end
if ~exist('UseOneInitDrawAtMode','var'), UseOneInitDrawAtMode=1; end

%% ------------------------------------------------------------------------

%% run chains

%% Display
fprintf('\n*****************************')
fprintf('\n* Generating MCMC sample(s) *')
fprintf('\n*****************************\n\n')

%% Set Timer
TimeStr = sprintf('MCMCUpdate%.0f',nUpdate);
TimeElapsed.(TimeStr) = toc;

%% Run MCMCFcn
if ~UseParallel
    for jChain=1:nChains
        fprintf('\nGenerating Chain %.0f of MCMC draws...\n',jChain)
        if UseOneInitDrawAtMode && jChain==1
            if exist('MCMCOptions','var'),MCMCOptions1 = MCMCOptions;end
            MCMCOptions1.x0 = [Params(:).postmode]';
            MCMCFcn(FileName.Post,...
                sprintf('%s%.0f',FileName.MCMCDraws,jChain),Post,nDraws,MCMCOptions1);
            clear MCMCOptions1
        else
            MCMCFcn(FileName.Post,...
                sprintf('%s%.0f',FileName.MCMCDraws,jChain),Post,nDraws,options{:});
        end
    end
else
    JobOptions = cell(nChains,1);
    for jChain=1:nChains
        if UseOneInitDrawAtMode && jChain==1
            if exist('MCMCOptions','var'),MCMCOptions1 = MCMCOptions;end
            MCMCOptions1.x0 = [Params(:).postmode]';
            JobOptions{jChain} = {FileName.Post,...
                sprintf('%s%.0f',FileName.MCMCDraws,jChain),Post,nDraws,MCMCOptions1};
            clear MCMCOptions1
        else
            JobOptions{jChain} = {FileName.Post,...
                sprintf('%s%.0f',FileName.MCMCDraws,jChain),Post,nDraws,options{:}};
        end
    end
    OutputPreamble = '\nChain %.0f:\n----------\n\n';
    vcSubmitQueuedJobs(nChains,@MCMCFcn,0,JobOptions,ListPathDependencies,nMaxWorkers,...
        'nMaxTasks',nMaxWorkers,'ShowOutput',1,'OutputPreamble',OutputPreamble,...
        'SaveTmpFileName',[FileName.Output,'_MCMCTmp'],...
        'LogFileName',MCMCLogFileName);
    clear JobOptions
end 
    
%% show rejection rates
for jChain=1:nChains
    eval(sprintf('load %s%.0f nDraws nRejections ScaleJumpFactor',FileName.MCMCDraws,jChain))
    fprintf('\nChain %02.0f: ScaleJumpFactor = %4.2f, rejection rate = %5.1f%%',...
        jChain,ScaleJumpFactor,nRejections/nDraws*100)
end
disp(' ')

%% ------------------------------------------------------------------------

%% Clean up
if ~isempty(MCMCLogFileName)
    for jChain=1:nChains
        delete(sprintf('%s%03.0f.log',MCMCLogFileName,jChain));
    end
end
clear options

%% Show time taken
TimeElapsed.(TimeStr) = toc-TimeElapsed.(TimeStr);
fprintf('\n%s %s\n\n',TimeStr,vctoc([],TimeElapsed.(TimeStr)))

%% ------------------------------------------------------------------------
