function MCMCFcn(PostName,SaveName,Post,nDraws,MCMCOptions)

% MaxPostFcn
%
% Maximizes the posterior by minimizing the negative of it.
%
% Usage:
%
% Out=MCMCFcn(PostName,SaveName,Post,nDraws)
% Out=MCMCFcn(PostName,SaveName,Post,nDraws,MCMCOptions)
%
% Inputs: [REQUIRED]
%   
%   PostName
%   Name of posterior evaluation function
%
%   SaveName
%   Name of the mat file where to save the results.
%
%   Post
%   Structure with the current information about the posterior. Needs
%   to have fields:
%     Post.Mode
%     Post.ModePost
%     Post.Var
%   
%   nDraws
%   number of draws
%
% Optional settings should be set in the structure MCMCOptions.
%
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost,
% MaxPostFcn, MakeTableMaxPost, MCMC, MCMCFcn, MCMCSearchScaleFactor,
% MakePlotsMCMCDraws, MCMCInference, MakeTableMCMCInference, 
% MakePlotsMCMCTrace, MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv,
% MakeReportMCMCPlots, MakeReportMCMC
%
% .........................................................................
%
% Created: March 27, 2008
% Updated: August 25, 2010
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% reset random number generator
s = clock;s = (s(6)-round(s(6)))*100000;rand('state',s);
s = clock;s = (s(6)-round(s(6)))*100000;randn('state',s);

%% set default options
ScaleJumpFactor = 2.4;
nIRS = 1000;
nBlocks = 1;
ExplosionScale = [.4,.5,.6,.7];
ExplosionProb = [.3,.6,.8,1];
nChangednIRS = 2;
InitDrawTStudent = 1;
InitDrawItMax = 1000;
InitDrawTol = 20;
InitDrawVarFactor = 1;
InitDrawDF = 4;
SaveList = {'xDraws','postDraws','nRejections','nDraws','ScaleJumpFactor','ScaleJump','VarJump'};

%% Update options
if exist('MCMCOptions','var')
    OptionFields = fieldnames(MCMCOptions);
    for jf=1:length(OptionFields)
        eval([OptionFields{jf},'=MCMCOptions.',OptionFields{jf},';'])
    end
end

%% preliminary calculations
np = length(Post.Mode);
if ~exist('isAugment','var'), isAugment = 0; end


%% ------------------------------------------------------------------------

%% generate initial draw
if ~isAugment && ~exist('x0','var')
    disp('Generating initial draw...')
    if InitDrawTStudent
        InitDrawVarChol = chol(InitDrawVarFactor^2*Post.Var)';
        InitDrawCorr = eye(np);
        for jd=1:InitDrawItMax
            x0 = Post.Mode+InitDrawVarChol*mvtrnd(InitDrawCorr,InitDrawDF)';
            post0=-feval(PostName,x0);
            if post0>Post.ModePost-InitDrawTol
                break
            end
        end
        if post0<Post.ModePost-InitDrawTol
            fprintf('Warning: Initial draw has low posterior density.\n')
        end
    else
        nExplosion = length(ExplosionScale);
        jChangednIRS = 0;tic
        while jChangednIRS<=nChangednIRS
            ExplosionDraws = unifrnd(0,1,1,nIRS);
            x0c = zeros(np,nIRS);
            x0c1 = zeros(np,nIRS);
            post0c = zeros(1,nIRS);
            post0c1 = zeros(1,nIRS);
        %     VarCholT = chol(0.5^2*Post.Var)';
            for jd=1:nIRS
                for je=1:nExplosion
                    if ExplosionDraws(jd)<=ExplosionProb(je)
                        ExpV = Post.Var*ExplosionScale(je)^2;
                        break
                    end
                end
                x0c(:,jd) = mvnrnd(Post.Mode,ExpV)';
                post0c(jd) = -feval(PostName,x0c(:,jd));
        %         x0c1(:,jd) = Post.Mode+VarCholT*mvtrnd(eye(np),x0DF)';
        %         post0c1(jd) = -feval(PostName,x0c1(:,jd));
            end
        %     post0cRaw = post0c;
            post0c = exp(post0c - repmat(Post.ModePost,1,nIRS));
            post0c = post0c./sum(post0c);
            [cpost0c,idx0]=sort(post0c);
            cpost0c = cumsum(cpost0c);
            pick0 = unifrnd(0,1);
            for j=1:nIRS
                if cpost0c(j)>=pick0
                    x0 = x0c(:,idx0(j));
                    % check whether the draw was ok...
                    WeightPick = post0c(idx0(j));
                    WeightMax = max(post0c);
                    fprintf('weight of the picked draw: %.6f\n',WeightPick)
                    fprintf('maximum weight for the draw: %.6f\n',WeightMax)
                    break
                end
            end
            if exist('x0','var')
                break
            else
                if jChangednIRS==nChangednIRS
                    x0 = Post.Mode;
                    WeightPick = NaN;
                    WeightMax = NaN;
                    fprintf('Warning: Did not find any suitable candidate after increasing nIRS. Using mode.\n')
                else
                    nIRS = nIRS*10;
                    jChangednIRS = jChangednIRS+1;
                    fprintf('Warning: Did not find any suitable candidate. Trying with increased nIRS.\n')
                end
            end
        end
    end
end

%% Prepare variables
if isAugment
    disp('Loading existing chain...')
    nDrawsNew = nDraws;
    load(SaveName)
    nDrawsOld = nDraws;
    nDraws = nDrawsNew;
    nDrawsToGen = nDraws-nDrawsOld;
    njj = ceil(nDrawsToGen/nBlocks);
    x0 = xDraws(:,end);
else
    nDrawsToGen = nDraws;
    njj = ceil(nDraws/nBlocks);
    ScaleJump = ScaleJumpFactor/np^(.5);
    VarJump = Post.Var*ScaleJump^2;
    xDraws=[];
    postDraws=[];
    nRejections = 0;
end

%% MCMC
% nj = nDraws/nBlocks;
% ScaleJump = ScaleJumpFactor/np^(.5);
% VarJump = Post.Var*ScaleJump^2;
% xDraws=[];
% postDraws=[];
% nRejections = 0;
post0=-feval(PostName,x0);
% for jj=1:nBlocks
%     fprintf('Generating set %2.0f out of %.0f...\n',jj,nBlocks)
%     xjj=[x0,zeros(np,nj)];
%     postjj=zeros(1,nj);
%     postjj(1)=post0;
%     clear x0;
for jj=1:nBlocks
    fprintf('Generating set %2.0f out of %.0f...\n',jj,nBlocks)
    nj = min(njj,nDrawsToGen-(jj-1)*njj);
    xjj = zeros(np,nj);
    postjj = zeros(1,nj);
    for j=1:nj
        xc = mvnrnd(x0,VarJump,1)';
        postc=-feval(PostName,xc);
        if unifrnd(0,1)<exp(postc-post0)
            x0 = xc;
            post0 = postc;
        else
            nRejections = nRejections+1;
        end
        xjj(:,j) = x0;
        postjj(j) = post0;
    end
    xDraws=[xDraws,xjj];
    postDraws=[postDraws,postjj];
    % save output
    save(SaveName,SaveList{:})
end

%% show number of rejections
% disp([int2str(nRejections),' rejections out of ',int2str(nDraws),' draws (',num2str(,'%).'])
fprintf('%.0f rejections out of %.0f draws (%.2f%%).\n',nRejections,nDraws,nRejections/nDraws*100)

%% ------------------------------------------------------------------------

%% save output
save(SaveName,SaveList{:})

%% ------------------------------------------------------------------------
