% MaxPost
%
% Runs several maximizations of the posterior
%
% Optional settings:
%
%   UseParallel
%   whether to use parallel or not. Default: false
%
%   nMax
%   Sets the number of maximizations, starting at different random points.
%   Default: 1
%
%   ShowOutput
%   If set to 2, it creates individual logs for each minimization and at
%   the end shows the summary of the minimization robustness in the main
%   output. If set to 1, the logs are not created, but it still shows the
%   robustness summary in the main output. If set to zero, none of these
%   are shown.
%   Default: 1
%
%   GuessOneAtMean
%   If set to 1, the first minimization uses the prior mean as the guess.
%   Others are drawn as usual (unless MinParams.Guess.x0 exists).
%   Default: 1
%
% See also:
% SetDSGE, GenSymVars, DataAnalysis, PriorAnalysis, GenPost, MaxPost,
% MaxPostFcn, MakeTableMaxPost, MCMC, MCMCFcn, MCMCSearchScaleFactor,
% MakePlotsMCMCDraws, MCMCInference, MakeTableMCMCInference, 
% MakePlotsMCMCTrace, MakePlotsMCMCPriorPost, MCMCConv, MakeTableMCMCConv,
% MakeReportMCMCPlots, MakeReportMCMC
%
% .........................................................................
%
% Created: March 25, 2008
% Updated: August 25, 2010
% by Vasco Curdia

%% ------------------------------------------------------------------------

%% Set options
if ~exist('UseParallel','var'),UseParallel = 0; end
if ~exist('nMax','var'),nMax = 1; end
if ~isfield(FileName,'TableMaxPost')
    FileName.TableMaxPost = [FileName.Output,'TableMaxPost'];
end
if ~exist('ShowOutput','var'),ShowOutput = 1;end
if ShowOutput==2
    LogName = [FileName.Output,'MaxPost_'];
else
    LogName = '';
end
if ~exist('GuessOneAtMean','var'),GuessOneAtMean=1;end
if ~exist('nMaxUsePrior','var'),nMaxUsePrior=0;end

%% settings for parallel computing
if UseParallel
    if ~exist('nMaxWorkers','var'), nMaxWorkers = 4; end
    if ~exist('nMaxTasks','var'), nMaxTasks = 1; end
end

%% check other optional inputs for MaxPostFcn
options = {};
if exist('MinParams','var'), options{end+1} = MinParams; end

%% ------------------------------------------------------------------------

%% run maximizations

%% Display
fprintf('\n****************************')
fprintf('\n* Maximizing the posterior *')
fprintf('\n****************************\n')

%% Set Timer
TimeStr = 'MaxPost';
TimeElapsed.(TimeStr) = toc;

%% Run minimizations
if ~UseParallel
    for jm=1:nMax
        if GuessOneAtMean && jm==1
            if exist('MinParams','var')
                MinParams1 = MinParams;
                if ~isfield(MinParams1,'Guess') || ~isfield(MinParams1.Guess,'x0Mean')
                    MinParams1.Guess.x0Mean = [Params(:).priormean]';
                end
            end
            MinParams1.Guess.x0 = MinParams1.Guess.x0Mean;
            MaxPostOut(jm) = MaxPostFcn(FileName.Post,Params,MinParams1);
            clear MinParams1
        elseif jm<=GuessOneAtMean+nMaxUsePrior
            if exist('MinParams','var'),MinParams1 = MinParams;end
            MinParams1.Guess.UsePrior = 1;
            MaxPostOut(jm) = MaxPostFcn(FileName.Post,Params,MinParams1);
            clear MinParams1
        else
            MaxPostOut(jm) = MaxPostFcn(FileName.Post,Params,options{:});
        end
        save([FileName.Output,'MaxPostTmp'],'MaxPostOut')
    end
else
    JobOptions = cell(nMax,1);
    for jm=1:nMax
        if GuessOneAtMean && jm==1
            if exist('MinParams','var'),MinParams1 = MinParams;end
            MinParams1.Guess.x0 = [Params(:).priormean]';
            JobOptions{jm} = {FileName.Post,Params,MinParams1};
            clear MinParams1
        elseif jm<=GuessOneAtMean+nMaxUsePrior
            if exist('MinParams','var'),MinParams1 = MinParams;end
            MinParams1.Guess.UsePrior = 1;
            JobOptions{jm} = {FileName.Post,Params,MinParams1};
            clear MinParams1
        else
            JobOptions{jm} = {FileName.Post,Params,options{:}};
        end
    end
    MaxPostOut = vcSubmitQueuedJobs(nMax,@MaxPostFcn,1,JobOptions,...
        ListPathDependencies,nMaxWorkers,'nMaxTasks',nMaxTasks,...
        'LogFileName',LogName,'ShowOutput',0,...
        'SaveTmpFileName',[FileName.Output,'_MaxPostTmp'],...
        'SaveTmpDelete',0);
    
    %% TEST
    save MaxPostTemp
    clear JobOptions
end
%MaxPostOut = [MaxPostOut{:}];
nMax = length(MaxPostOut);

%% ------------------------------------------------------------------------

%% If desired show history evolution of robustness
if ShowOutput>0 && UseParallel
    for jm=1:nMax
        Outj = MaxPostOut(jm).MinOutput;
        fprintf('\nRobustness analysis for minimization %3.0f',jm)
        fprintf('\n----------------------------------------\n')
        fprintf('Iteration %3.0f: function value: %15.8f',0,Outj(1).fh)
        fprintf('         %15s','')
        fprintf(' stopped at iteration %.0f\n',Outj(1).itct)
        itbest.fh = Outj(1).fh;
        itbest.idx = 1;
        for jr=2:length(Outj)
            fprintf('Iteration %3.0f: function value: %15.8f',jr-1,Outj(jr).fh)
            itchg = Outj(jr).fh-itbest.fh;
            fprintf(' change: %15.8f',itchg)
            fprintf(' stopped at iteration %.0f\n',Outj(jr).itct)
            if itchg<0
                itbest.fh = Outj(jr).fh;
                itbest.idx = jr;
            end
        end
        fprintf('Best iteration: %.0f\n',itbest.idx-1)
        fprintf('Best iteration function value: %.8f\n',itbest.fh)
        fprintf('Robustness change over initial min: %.8f\n',itbest.fh-Outj(1).fh)
        fprintf('Robustness message: %s\n',MaxPostOut(jm).RrcMsg)
        fprintf('Best iteration message: %s\n\n',Outj(itbest.idx).rcMsg)
    end
end
clear Outj jm itbest itchg jr

%% ------------------------------------------------------------------------

%% show results for each maximization
fprintf('\nIndividual maximizations (ordered from best to worst):')
fprintf('\n======================================================\n\n')
namelength = [cellfun('length',{Params(:).name})];
% show headers
str2show = '     log-density';
for jp=1:np
    str2show = sprintf(['%s %',int2str(max(namelength(jp),9)),'s'],...
        str2show,Params(jp).name);
end
disp(str2show)
% order maximizations
[SortPost,idxSortPost] = sort([MaxPostOut(:).f]);
% show results
for jm=1:nMax
    jShow = idxSortPost(jm);
    str2show = sprintf('%4.0f %11.4f',jShow,-MaxPostOut(jShow).f);
    for jp=1:np
        str2show = sprintf(['%s %',int2str(max(namelength(jp),9)),'.4f'],...
            str2show,MaxPostOut(jShow).x(jp));
    end
    disp(str2show)
end
disp(' ')

%% extract the best one
[Post.ModePost, idxMax] = min([MaxPostOut(:).f]);
Post.ModePost = -Post.ModePost;
Post.Mode = MaxPostOut(idxMax).x;
Post.Var = MaxPostOut(idxMax).H;

%% update Params
for j=1:np
    Params(j).postmode = Post.Mode(j);
    Params(j).postse = Post.Var(j,j)^(1/2);
end

%% display results on screen
fprintf('\nResults from maximization of posterior:')
fprintf('\n=======================================\n')
namelength = [cellfun('length',{Params(:).name})];
namelengthmax = max(namelength);
DispList = {'','','name';
            'Prior','dist','priordist';
            '','  mode','priormode';
            '','  mean','priormean';
            '','   se','priorse';
            '','   5%','priorp050';
            '',' median','priorp500';
            '','   95%','priorp950';
            'Posterior',' mode ','postmode';
            '','  se','postse';
            }';
nc = size(DispList,2);
for jr=1:2
    str2show = sprintf(['%-',int2str(namelengthmax),'s'],DispList{jr,1});
    str2show = sprintf('%s  %-4s',str2show,DispList{jr,2});
    for jc=3:nc
        str2show = sprintf('%s  %-7s',str2show,DispList{jr,jc});
    end
    disp(str2show)
end
for j=1:np
    str2show = sprintf(['%',int2str(namelengthmax),'s'],Params(j).(DispList{3,1}));
    str2show = sprintf('%s  %4s',str2show,Params(j).(DispList{3,2}));
    for jc=3:nc
        str2show = sprintf('%s  %7.4f',str2show,Params(j).(DispList{3,jc}));
    end
    disp(str2show)
end
fprintf('\nposterior log density at mode: %.6f\n\n',Post.ModePost)

%% save minimization output
% no need to be saved with everything else
save([FileName.Output,'MaxPostOut'],'MaxPostOut','idxMax')
clear MaxPostOut idxMax

%% clean up
clear str2show options

%% create tex table
MakeTableMaxPost(FileName.TableMaxPost,Params,Post)

%% Show time taken
TimeElapsed.(TimeStr) = toc-TimeElapsed.(TimeStr);
fprintf('\nMaxPost %s\n\n',vctoc([],TimeElapsed.(TimeStr)))

%% ------------------------------------------------------------------------
