% BCL_SignBoot2 - ## Bootstrap Confidence Band for BCL (pre & post Euro Sample) 
% 
% [CC0,CC1,CC2] = BCL_SignBoo2(X,cd,q,r,p,h,L,nrest,rest,ndraws,horiz,maxrot)
% 
% Inputs:
%    CC0 - Impulse responses whole sample
%    CC1 - Impulse responses pre 1998
%    CC2 - Impulse responses post 1999
% 
% Outputs:
%      X - Data                                                             (1)
%     cd - code for data transformation                                     (2)
%      q - n of dynamic factors                                            (3)
%      r - n of stati factors                                              (4)
%      p - n of lags in the VAR                                            (5)
%      h - n of lags in the MA representation                              (6)
%  nrest - normalization restrictions                                       (8)
%   rest - restrictions                                                     (9)
% ndraws - n of draws for the bootstrap algorithm                          (10)
%  horiz - n of horizon at which the restriction is imposed                (11)
% nrepli - n of replications                                               (12)
% maxrot - Maximum Number of Rotation                                       (13) optional
%

% Original version by Mario Forni
% modified by Matteo Luciani - mluciani@ulb.ac.be

function [CC0,CC1,CC2,NGR] = BCL_SignBoot2(x,cd,q,r,p,h,nrest,rest,ndraws,horiz,nboot,maxrot)

inizio=clock;                                                               % When the procedure starts
N = size(x,2);                                                              % Number of Variables
WW = diag(std(x)); WWW=repmat(diag(WW),1,q);                                % Standard deviation
y = ML_center(x)*(WW^-1);                                                   % standardize variables
NGR=NaN*ones(nboot,3);                                                      % preallocates variables
if  isempty(maxrot); maxrot = 200; end                                      % Maximum Number of Rotation
det=1;                                                                      % by default a constant is included
[fhat,lambda0]=ML_efactors2(y,r,2);                                         % Estimating Static Factors with the normalization lambda'*lamnda/N=I
[~,~,~, Lambda] = ML_factor_loadings_break(y,fhat,63);                      % Estimation of factor loadings when there is a breakpoint
lambda1=Lambda(:,:,1);                                                      % Loadings pre-1999
lambda2=Lambda(:,:,2);                                                      % Loagings post-1999
[A u]=ML_VAR(fhat,p,det);                                                   % Estimating the Law of Motion fo the Static Factors   
[eta G]=ML_edynfactors2(u,q);                                               % Estimating Dynamic Factors Innovation
A2=ML_VARbias(fhat,u,A,p,det,nboot,eta,G');                                 % Computes the bias induced by VAR Estimation - Kilian (1998)
n=size(u,1);

tic
for bb=1:nboot;        
    bootsam=ceil(n*rand(n,1));                                              % reshuffling residuals    
    fstar=ML_decomposition(fhat,A2,eta(bootsam,:),det,G');                  % Generates Static Factors
    [Astar ustar]=ML_VAR(fstar,p,det);                                      % Estimating the Law of Motion fo the Static Factors   
    [~, Gstar]=ML_edynfactors2(ustar,q);                                    % Estimating Dynamic Factors Innovation
    Bstar = ML_MA(h,Astar,det);                                             % MA Representation of the static factors
    for ii=1:h; 
        CLstar0(:,:,ii)=(lambda0*Bstar(:,:,ii)*Gstar).*WWW;                 % MA Representation of common components
        CLstar1(:,:,ii)=(lambda1*Bstar(:,:,ii)*Gstar).*WWW;                 % MA Representation of common components
        CLstar2(:,:,ii)=(lambda2*Bstar(:,:,ii)*Gstar).*WWW;                 % MA Representation of common components
    end;             
    BB0 = CumImp(CLstar0, cd);                                              % Comulates IRF if necessary
    BB1 = CumImp(CLstar1, cd);                                              % Comulates IRF if necessary
    BB2 = CumImp(CLstar2, cd);                                              % Comulates IRF if necessary
    
    GR0 = ML_GoodPolar(BB0,nrest,rest,ndraws,horiz,maxrot);                 % Good Rotations
    GR1 = ML_GoodPolar(BB1,nrest,rest,ndraws,horiz,maxrot);                 % Good Rotations
    GR2 = ML_GoodPolar(BB2,nrest,rest,ndraws,horiz,maxrot);                 % Good Rotations
    
    C0 = ML_ComputeIrf(BB0,GR0);                                            % Compute IRF   
    C1 = ML_ComputeIrf(BB1,GR1);                                            % Compute IRF   
    C2 = ML_ComputeIrf(BB2,GR2);                                            % Compute IRF   
    
    clear fstar Astar ustar Gstar Bstar BB0 BB1 BB2                         % Clear Objects to free space in the memory
    
    ngr0=size(GR0,3); ngr1=size(GR1,3); ngr2=size(GR2,3);                   % Number of Good Rotations
    
    if ngr0 ~= 0; 
        NGR(bb,1)=1;                                                        % Identifies how many, and which bootstrap yeld at least one rotation
        if ngr0 > 1;                    
            BR0=ML_SignRestrictionFryPagan(C0,rest);                        % Identify the best rotation in the sense of Fry & Pagan                                           
            C0=C0(:,:,:,BR0);                                               % Keep Only the FP rotation     
        end       
    end;     
    if ngr1 ~= 0; 
        NGR(bb,2)=1;                                                        % Identifies how many, and which bootstrap yeld at least one rotation
        if ngr1 > 1;                    
            BR1=ML_SignRestrictionFryPagan(C1,rest);                        % Identify the best rotation in the sense of Fry & Pagan                                           
            C1=C1(:,:,:,BR1);                                               % Keep Only the FP rotation     
        end       
    end;             
    if ngr2 ~= 0; 
        NGR(bb,3)=1;                                                        % Identifies how many, and which bootstrap yeld at least one rotation
        if ngr2 > 1;                    
            BR2=ML_SignRestrictionFryPagan(C2,rest);                        % Identify the best rotation in the sense of Fry & Pagan                                           
            C2=C2(:,:,:,BR2);                                               % Keep Only the FP rotation     
        end       
    end;     
    CC00{bb}=C0;
    CC01{bb}=C1;
    CC02{bb}=C2;   
    
    disp([nboot-bb ngr0 ngr1 ngr2 toc])                                     % Counter
    tic
end

clear GR0 GR1 GR2 C0 C1 C2

for bb=1:nboot; tgr(bb)=size(CC00{bb},4); end; tgr=[0 tgr];                                              % idemtifies how many rotations we have;
CC0=zeros(N,q,h,sum(tgr)); for bb=1:nboot; CC0(:,:,:,sum(tgr(1:bb))+1:sum(tgr(1:bb+1)))=CC00{bb}; end    % convert from cell to matrix

for bb=1:nboot; tgr(bb)=size(CC01{bb},4); end; tgr=[0 tgr];                                              % idemtifies how many rotations we have;
CC1=zeros(N,q,h,sum(tgr)); for bb=1:nboot; CC1(:,:,:,sum(tgr(1:bb))+1:sum(tgr(1:bb+1)))=CC01{bb}; end    % convert from cell to matrix

for bb=1:nboot; tgr(bb)=size(CC02{bb},4); end; tgr=[0 tgr];                                              % idemtifies how many rotations we have;
CC2=zeros(N,q,h,sum(tgr)); for bb=1:nboot; CC2(:,:,:,sum(tgr(1:bb))+1:sum(tgr(1:bb+1)))=CC02{bb}; end    % convert from cell to matrix

fine=clock;                                                                 % When the procedure ends
format bank; disp('__________________________________________________________');
fprintf('Procedure startet at:'); disp(inizio(4:end));
fprintf('Procedure ended at:  '); disp(fine(4:end));
disp(''); format