% BLL_SignBootVAR2 - Bootstrap Confidence band for NS-DFM wih short-long restrictions
% 
% [CC,eflag] = BLL_SLRBootVAR2(Y,cd,q,r,p,h,ID,K0,K1,nboot,lr,J,BaB)
% 
% Outputs:
%     CC - Impulse responses     
% 
% Inputs:
%      Y - I(1) Data 
%     cd - code for data transformation
%      q - n of common shocks
%      d - n of transitory shocks
%      r - n of common factors
%      p - n of lags in the VAR       
%      h - n of lags in the MA representation
%     ID - variables on which restrictionms are imposed
%     KO - Short run restrictions
%     K1 - Long Run Restrictions
%     lr - if 1 ==> LR restrictions imposed on Phi(40) (optional)
%      J - Variables that need to be detrended (optional)
%    BaB - if 1 ==> boostrap after bootstrap (optional)


% Matteo Luciani (matteoluciani@yahoo.it)


function [CC,eflag] = BLL_SLRBootVAR2(Y,cd,q,r,p,h,ID,K0,K1,nboot,lr,J,BaB)
warning off;
try lr; catch; lr=0; end
if lr==1; h1=40; else h1=h; end

y=ML_diff(Y);                                                               % 1st Differences
[yy, ~, sy]=ML_Standardize(y);                                              % Standardize
try J2=find(J); % ------------------------------------------------------------------ % Transform variables in levels 
    X=ML_center(Y);                                                         % center variables in levels
    X(:,J2)=ML_detrend(Y(:,J2),1);                                            % Detrend variables with linear trend
catch; X=ML_detrend(Y,1);                                                   % Detrend all variables
end                                                                         % ---------------------
Z=X./repmat(sy,size(X,1),1);                                                % BLL - Standardization 
[~, N]=size(y);                                                             % Panel Dimensions
[~,lambda]=ML_efactors2(yy,r,2);                                            % estimate factors in first differences
F=Z*lambda/N;                                                               % BLL Factors
[AA,v,AL,B]=ML_VAR(F,p,1,h);                                                % Estimating VAR on Factors   
[V, D] = eigs(cov(v),q,'lm'); G=V*(D^.5); u=v*V*(D^-.5);                    % common shocks
vv=ML_center(u*G');                                                         % rxr residuals with rank q
n=size(u,1);
det=1;



%%% Bootstrap After Botstrap Correction %%%
try
    if BaB==1;
        n=size(v,1);
        psi=nan(size(AA,1),size(AA,2),nboot);
        fprintf('Computing Correction for Bootsrap After Bootsrap')
        for bb=1:nboot;
            bootsam=ceil(n*rand(n,1));                                          % reshuffling residuals
            Fstar=ML_GenerateVAR(F,AL,vv(bootsam,:),det,p,AA(1,:)');            % Generates Static Factors
            AAstar=ML_VAR(Fstar,p,1,h);                                         % Estimating VAR on Factors
            psi(:,:,bb)=AAstar-AA;
        end
        AA2=AA-mean(psi,3);
        disp(' - done')
        
        Atilda0=ML_VAR_companion_matrix(ML_VAR_polynomial(AA,det));             % VAR in companion form
        mu0=max(eig(Atilda0)); 
        
        Atilda=ML_VAR_companion_matrix(ML_VAR_polynomial(AA2,det));             % VAR in companion form
        mu=max(eig(Atilda));                                                        % Max Root of the VAR
        for pp=1:p; 
%             AA2(det+1+pp-1:p:end,:)=AA2(det+1+pp-1:p:end,:)/(mu^pp);
            AA2(det+1+pp-1:p:end,:)=(mu0^pp)*AA2(det+1+pp-1:p:end,:)/(mu^pp); 
        end    % Correct coefficients so that max root is 1
        AA=A2;
        AL=ML_VAR_polynomial(AA2,det);
    end
catch
end

%%% Bootsrap Algorithm %%%
%matlabpool size;
%if ans==0; matlabpool; end

tic
fprintf('Running Bootsrap Algorithm')
WW=repmat(sy',1,q);
CC=nan(N,q,h,nboot);                                                        % Preallocates Variables
for bb=1:nboot;        
    [CC(:,:,:,bb), eflag(:,bb)]=aux1(n,F,AA,vv,p,q,h,lambda,WW,cd,K0,K1,ID,det,h1,AL);
end
disp(' - done')
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [CC, eflag]=aux1(n,F,AA,vv,p,q,h,lambda,WW,cd,K0,K1,ID,det,h1,AL)

bootsam=ceil(n*rand(n,1));                                                  % reshuffling residuals  
Fstar=ML_GenerateVAR(F,AL,vv(bootsam,:),det,p,AA(1,:)');                    % Generates Static Factors
[~,vstar,~,CLstar]=ML_VAR(Fstar,p,1,h);                                     % Estimating VAR on Factors
[~, Gstar]=ML_edynfactors2(vstar,q);                                        % common shocks
for jj=1:h; Phistar(:,:,jj)=(lambda*CLstar(:,:,jj)*Gstar).*WW; end;         % Impulse Responses
BB = CumImp(Phistar,cd);                                                    % comulate IRFs if it is necessary
[H, eflag]=ML_SLRrestrictions(ID,BB,BB(:,:,h1),K0,K1);                      % Rotation Matrix
CC = ML_ComputeIrf(BB,H);                                                   % Compute IRF

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CC = CumImp(Imp, Transf)
notransf = find(Transf==0);
firstdiff = find(Transf==1);
seconddiff = find(Transf==2);
CC = Imp*0;
CC(notransf,:,:,:) = Imp(notransf,:,:,:);
CC(firstdiff,:,:,:) = cumsum(Imp(firstdiff,:,:,:),3);
CC(seconddiff,:,:,:) = cumsum(cumsum(Imp(seconddiff,:,:,:),3),3);





