% BLL_SignBoot - Bootstrap Confidence band for I(1) SDFM wih sign restrictions
% 
% [CC,u] = BLL_SignBoot(Y,cd,q,d,r,p,h,nrest,rest,ndraws,horiz,nboot,maxrot,FP)
% 
% Outputs:
%     CC - Impulse responses
%      u - number of boostrap which finds at least one rotation
% 
% Inputs:
%      Y - I(1) Data                                                        (1)
%     cd - code for data transformation                                     (2)
%      q - n of common shocks
%      d - n of transitory shocks
%      r - n of common factors
%      p - n of lags in the VAR                                            (5)
%      h - n of lags in the MA representation                              (6)
%  nrest - normalization restrictions                                       (7)
%   rest - restrictions                                                     (8)
% ndraws - n of draws for the bootstrap algorithm                          (9)
%  horiz - n of horizon at which the restriction is imposed                (10)
% nrepli - n of replications                                               (11)
% maxrot - Maximum Number of Rotation                                       (12) optional
%
% Written by Matteo Luciani (matteo.luciani@frb.gov)
% 
% Reference: Barigozzi, M, Lippi, M. and Luciani M. (2016) "Non-Stationary 
%            Dynamic Factor Models for Large Datasets", FEDS 2016-024
% 

function [CC,u, BB] = BLL_SignBoot2(Y,cd,q,d,r,p,h,nrest,rest,ndraws,horiz,nboot,maxrot)

tic

if nargin<13; maxrot=ndraws; end;

y=ML_diff(Y);                                                               % 1st Differences
[yy, ~, sy]=ML_Standardize(y);                                              % Standardize
X=ML_detrend(Y);                                                            % Detrend Data
Z=X./repmat(sy,size(X,1),1);                                                % BLL - Standardization 
[T, N]=size(y);                                                             % Panel Dimensions
[~,lambda]=ML_efactors2(yy,r,2);                                            % estimate factors in first differences
F=Z*lambda/N;                                                               % BLL Factors
[AA, ~, ~, ~, ~, v]=BLL_VECM(F,p,r-q+d,h,1);                                % Estimate the VECM
[V, D] = eigs(cov(v),q,'lm'); G=V*(D^.5); u=v*V*(D^-.5);                    % common shocks
vv=u*G';
n=size(u,1);

CC=nan(N,q,h,nboot); BB=CC;             % Preallocates Variables

matlabpool size;
if ans==0; matlabpool; end
%%% Bootsrap Algorithm %%%
disp('starting bootstrap algorithm')
J=zeros(nboot,1);
parfor bb=1:nboot;        
   [t1, t2]=aux1(n,F,AA,vv,p,r,q,d,h,lambda,sy,cd,nrest,rest,ndraws,horiz,maxrot);
      if isempty(t1);
       J(bb)=1;
   else
       CC(:,:,:,bb)=t1;
       BB(:,:,:,bb)=t2;
   end;
end
J2=sum(J);
if J2>0;
    disp(['Number of unsuccesfull draws = ' num2str(J2)])
    CC(:,:,:,find(J))=[];
    BB(:,:,:,find(J))=[];
end
disp('done')
toc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CC = CumImp(Imp, Transf)
notransf = find(Transf==0);
firstdiff = find(Transf==1);
seconddiff = find(Transf==2);
CC = Imp*0;
CC(notransf,:,:,:) = Imp(notransf,:,:,:);
CC(firstdiff,:,:,:) = cumsum(Imp(firstdiff,:,:,:),3);
CC(seconddiff,:,:,:) = cumsum(cumsum(Imp(seconddiff,:,:,:),3),3);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [C,B]=aux1(n,F,AA,vv,p,r,q,d,h,lambda,sy,cd,nrest,rest,ndraws,horiz,maxrot)

bootsam=ceil(n*rand(n,1));                                                          % reshuffling residuals
Fstar=ML_decomposition(F,AA,vv(bootsam,:),1);                                    % Generates Static Factors
[~, ~, ~, CLstar, ~, vstar]=BLL_VECM(Fstar,p,r-q+d,h,1);                            % Estimate the VECM
[V, D] = eigs(cov(vstar),q,'lm'); Gstar=V*(D^.5);                                   % common shocks
for jj=1:h; Phistar(:,:,jj)=(lambda*CLstar(:,:,jj)*Gstar).*repmat(sy',1,q); end;	% Impulse Responses
PHIstar = CumImp(Phistar,cd);                                                       % comulate IRFs if it is necessary
GR = ML_GoodPolar(PHIstar,nrest,rest,ndraws,horiz,maxrot);                          % Good Rotations
C = ML_ComputeIrf(PHIstar,GR); B = ML_ComputeIrf(Phistar,GR);                       % Compute IRF

ngr=size(GR,3);
if ngr ~= 0;    
    if ngr>1;
        BR=ML_SignRestrictionFryPagan(C,rest);                                      % Identify the best rotation in the sense of Fry & Pagan
        C=C(:,:,:,BR); B=B(:,:,:,BR);                                         % Keep Only the FP rotation
    end    
end;
